#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

static char *filetype = "?pc?d?b?-?l?s";

static char *perm_chars = "rwxsStT";

static int perm_flags[] = {
	S_IRUSR, S_IWUSR, S_IXUSR, S_ISUID,
	S_IRGRP, S_IWGRP, S_IXGRP, S_ISGID,
	S_IROTH, S_IWOTH, S_IXOTH, S_ISVTX
};

char *
mode_to_perms(char *buf, mode_t mode)
{
	int i, j, n;
	char *ret;

	ret = buf;

	n = (mode & S_IFMT) >> 12;

	buf[0] = filetype[n];

	++buf;

	for (i = 0; i < 3; i++) {

		for (j = 0; j < 3; j++) {
			n = i*4 + j;
			if (mode & perm_flags[n])
				buf[j] = perm_chars[j];
			else
				buf[j] = '-';
		}

		if (mode & perm_flags[n+1]) {
			n = j + (mode & perm_flags[n] ? 0 : 1) + (i == 2 ? 2 : 0);
			buf[j-1] = perm_chars[n];
		}

		buf += 3;
	}

	return ret;
}

#define N_ELEMS(a)	(sizeof(a)/sizeof(a[0]))

int
main(int argc, char *argv[])
{
	int i, res;
	struct stat st;
	float size;
	char *p, pmode[11];
	char *suffix[] = { "G", "M", "K" };
	int sztab[] = { 1 << 30, 1 << 20, 1 << 10 };

	if (argc < 2) {
		printf("Usage: %s file\n", argv[0]);
		exit(1);
	}

	res = stat(argv[1], &st);

	if (res == -1) {
		perror("stat()");
		exit(1);
	}

	p = NULL;

	for (i = 0; i < N_ELEMS(sztab); i++) {
		if (st.st_size > sztab[i]) {
			size = (float) st.st_size / sztab[i];
			p = suffix[i];
			break;
		}
	}

	mode_to_perms(pmode, st.st_mode);

	pmode[sizeof(pmode)-1] = '\0';

	printf("%-60.60s %s %4.1f%s\n", argv[1], pmode, size, p != NULL ? p : "");

	return 0;
}

